#!/usr/bin/env node


/* eslint-disable no-tabs */
'use strict';

let handleStarring = (() => {
	var _ref = _asyncToGenerator(function* (token, packageNames, starAll = false) {
		const packageDetails = yield getPackageDetails(token, packageNames);
		const starredName = packageDetails.filter(function (pd) {
			return pd.starred;
		}).map(function (pd) {
			return pd.name;
		});
		const unStarredPackages = packageDetails.filter(function (pd) {
			return !pd.starred;
		});

		if (!isEmpty(starredName)) {
			if (starAll) {
				console.log(colors.yellow.bold(`👍  You have already starred ${colors.cyan(arrayToSentence(starredName))}`));
			} else {
				console.log(colors.yellow.bold(`👍  You have already starred${isEmpty(unStarredPackages) ? ' all the' : ''} ${starredName.length} ${pluralize('dependency', starredName.length)}.\n`));
			}
		}

		if (isEmpty(unStarredPackages)) {
			return;
		}

		if (isNpm) {
			starNpmPackage(token, unStarredPackages);
			return;
		}

		if (cli.flags.all || starAll) {
			starRepos(token, unStarredPackages);
		} else {
			selectSomePackages(token, unStarredPackages);
		}
	});

	return function handleStarring(_x, _x2) {
		return _ref.apply(this, arguments);
	};
})();

let starGlobalPackages = (() => {
	var _ref2 = _asyncToGenerator(function* (token) {
		try {
			const globalPkgs = yield globalPackages();
			handleStarring(token, globalPkgs);
		} catch (err) {
			console.log(err);
		}
	});

	return function starGlobalPackages(_x3) {
		return _ref2.apply(this, arguments);
	};
})();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const meow = require('meow');
const ghAuth = require('ghauth');
const loadPkg = require('read-pkg-up');
const updateNotifier = require('update-notifier');
const inquirer = require('inquirer');
const isNpm = require('is-npm');
const isEmpty = require('just-is-empty');
const arrayToSentence = require('array-to-sentence');
const colors = require('colors/safe');
const globalPackages = require('global-packages');
const pluralize = require('pluralize');
const pkg = require('../package.json');
const starRepos = require('./star-repos');
const getPackageDetails = require('./get-package-details');
const selectSomePackages = require('./select-some-packages');

updateNotifier({ pkg }).notify();

function getPackageNames(pkg) {
	const deps = Object.assign({}, pkg.dependencies, pkg.devDependencies);
	return Object.keys(deps);
}

const cli = meow(`
	${colors.green('Usage')}
	  $ ${colors.bold('starring')} ${colors.yellow('[input]')}

	${colors.green('Options')}
	  ${colors.bold.yellow('--all')}  Star all the local packages listed in the package.json from current project. [Default: false]
		${colors.bold.yellow('--global')} Star global packages.

	${colors.green('Examples')}
	  $ ${colors.yellow.bold('starring')}
	  presents UI to select the packages you want to star.
	  $ ${colors.yellow.bold('starring colors')}
	  stars colors npm package on GitHub.
`);

function init(token) {
	loadPkg().then(({ pkg }) => {
		if (!pkg) {
			console.log(colors.red.bold('✖ No package.json found. Ensure that you are inside the project directory.'));
			return;
		}
		const packageNames = getPackageNames(pkg);
		handleStarring(token, packageNames);
	});
}

const opts = {
	configName: 'starring',
	scopes: ['public_repo'],
	note: 'Star github repos',
	userAgent: 'starring'
};

function starNpmPackage(token, packageDetails) {
	inquirer.prompt({
		type: 'confirm',
		name: 'confirmStar',
		message: `Would you like to star ${arrayToSentence(packageDetails.map(pd => pd.name))} ?`
	}).then(answers => {
		if (answers.confirmStar) {
			starRepos(token, packageDetails);
		}
	});
}

ghAuth(opts, (() => {
	var _ref3 = _asyncToGenerator(function* (err, { token }) {
		if (err) {
			console.log(err);
		}

		if (!isEmpty(cli.input)) {
			handleStarring(token, cli.input, true);
			return;
		}

		if (cli.flags.global) {
			starGlobalPackages(token);
			return;
		}

		init(token);
	});

	return function (_x4, _x5) {
		return _ref3.apply(this, arguments);
	};
})());