'use strict';

let getPackageDetails = (() => {
	var _ref = _asyncToGenerator(function* (token, packageNames) {
		const spinner = ora(`Fetching details from npm and GitHub`).start();

		const repoNames = packageNames.map(function (packageName) {
			return got(`https://api.npms.io/v2/package/${packageName}`, { json: true });
		});
		const results = yield pSettle(repoNames);

		const details = decorateDetails(packageNames, results);

		const mapper = details.map(function ({ repo }) {
			return ghGot(`user/starred/${repo}`, { token, method: 'GET' });
		});
		const starResults = yield pSettle(mapper);

		spinner.succeed('Details Fetched from npm and GitHub.');

		details.forEach(function (d, i) {
			d.starred = starResults[i].isFulfilled && starResults[i].value.statusCode === 204;
		});
		return details;
	});

	return function getPackageDetails(_x, _x2) {
		return _ref.apply(this, arguments);
	};
})();

function _asyncToGenerator(fn) { return function () { var gen = fn.apply(this, arguments); return new Promise(function (resolve, reject) { function step(key, arg) { try { var info = gen[key](arg); var value = info.value; } catch (error) { reject(error); return; } if (info.done) { resolve(value); } else { return Promise.resolve(value).then(function (value) { step("next", value); }, function (err) { step("throw", err); }); } } return step("next"); }); }; }

const pSettle = require('p-settle');
const got = require('got');
const ghGot = require('gh-got');
const ora = require('ora');
const parseUrl = require('parse-github-url');

function decorateDetails(packageNames, details) {
	return details.map((detail, i) => {
		const x = {
			name: packageNames[i],
			success: detail.isFulfilled
		};

		if (detail.value) {
			var _detail$value$body$co = detail.value.body.collected.metadata;
			const links = _detail$value$body$co.links,
			      author = _detail$value$body$co.author,
			      description = _detail$value$body$co.description;


			x.description = description;
			x.repo = Boolean(links.repository) && parseUrl(links.repository).repo;
			x.author = author && author.name;
		}

		return x;
	});
}

module.exports = getPackageDetails;